#!/usr/bin/ruby -w

require 'setUpExperimentSpecifications'

def logExecution()
  f = File.open(Socket.gethostname().gsub!(/[[:space:].]/,"")+".log","a")
  f.puts(" [" + Time.new().to_s() + "] " +\
         File.basename($0) + " " + $*.join(" "))
  f.close
end

def runNS2(protocol,threshold,
           numb_nodes,width,height,interval,
           positionFile,trafficFile,outFile,specname)

  nsCommand = 
    "time nice ns test.tcl        -traf     \"#{positionFile}\" "             +\
    "-pos \"#{trafficFile}\" -optpampa   #{protocol} -optargs \"3e6 #{threshold}\" "+\
    "-nn    #{numb_nodes}    -x          #{height}   -y   #{width} "     +\
    "-stop  #{interval*numb_nodes+1000}              -out #{outFile}"
#  puts   nsCommand
  puts "--> Running ns-2 for #{resultString(protocol,threshold,specname)}"
  system nsCommand
  if (!$?) then 
    raise "ns2 had some problem"
  end
  puts
end

EXPERIENCE_RESULTS_DIRECTORY="EXP_RESULTS"

def resultString(protocol,threshold,specname)
  if (threshold!=2) then
    return "P-#{protocol}-THR#{threshold}-#{specname}"
  else
    return "P-#{protocol}-#{specname}"
  end
end

def experimentName(protocol,threshold,specname)
  return "#{getTimeStamp()}-#{resultString(protocol,threshold,specname)}"
end

def runExperimentSpec(protocol,threshold,nn,positionFile,trafficFile,specName,directory)

  f = File.open(trafficFile, "r")
  firstLine = f.gets
  width     = firstLine.scan(   /WIDTH: ([0-9.]*),/).first.first.to_i
  height    = firstLine.scan(  /HEIGHT: ([0-9.]*),/).first.first.to_i
  interval  = firstLine.scan(/INTERVAL: ([0-9.]*),/).first.first.to_i
  f.close

  basename  = experimentName(protocol,threshold,specName)
  outFile  = "#{$DIR_PREFIX}#{directory}/#{basename}"
  runNS2(protocol,threshold,nn,width,height,interval,
         positionFile,trafficFile,outFile,specName)
  traceFile  = "#{basename}.tr"
  system "mv #{$DIR_PREFIX}#{directory}/#{traceFile} "+\
    "        #{$DIR_PREFIX}#{directory}/NO-UNISON-TRACES/"
  if (!$?) then 
    raise "Could not move #{$DIR_PREFIX}#{directory}/#{traceFile}"
  end
  system "gzip #{$DIR_PREFIX}#{directory}/NO-UNISON-TRACES/#{traceFile}"
  if (!$?) then 
    raise("Could not gzip #{$DIR_PREFIX}#{directory}"+\
          "/NO-UNISON-TRACES/#{traceFile}")
  end
  return "#{outFile}.stats"
end

def getExperimentResults(protocol,threshold,
                         topo,w,h,b,n,i,id,phases,
                         directory=EXPERIENCE_RESULTS_DIRECTORY)

  provideDirectory("#{$DIR_PREFIX}#{directory}")
  provideDirectory("#{$DIR_PREFIX}#{directory}/NO-UNISON-TRACES")

  (nn,positionFile,trafficFile,specName) = 
    getExperimentSpecFiles(topo,w,h,b,n,i,id,phases)

  statsFileS = 
    Dir.glob("#{$DIR_PREFIX}#{directory}/*"+\
             "#{resultString(protocol,threshold,specName)}.stats")
  traceFileS = 
    Dir.glob("#{$DIR_PREFIX}#{directory}/NO-UNISON-TRACES/*"+\
             "#{resultString(protocol,threshold,specName)}.tr.gz")
  case
  when ((statsFileS.length==0) and (traceFileS.length==0)) then
    # *** NONE EXISTS
    if phases.include?("-ns2")
      statsFile = 
        runExperimentSpec(protocol,threshold,nn,
                          positionFile,trafficFile,specName,directory)
    else
      statsFile = nil
      if phases.include?("-analysis")
        raise("Cannot do analysis: stats file missing "+\
              "for #{resultString(protocol,threshold,specName)}. "+\
              "Allow -ns2 phase to solve this")
      end
    end
  when (statsFileS.length==1) then
    # *** STATSFILE EXIST
    statsFile = statsFileS.first
  else
    raise("Trace file with no stats file!")
  end
  return statsFile
end

def extractCoverageAndCostRatioFromStatsFile(statsFile)
  totalRecvm  = 0
  totalTsentm = 0
  nbNodes     = 0
  File.new(statsFile).each_line do|line|
    totalRecvm  += line.scan(/-recvm ([0-9]*) / ).first.first.to_i + 1
    totalTsentm += line.scan(/-tsentm ([0-9]*) /).first.first.to_i
    nbNodes+=1
  end # through all ines of the current stat file
  return [totalRecvm*1.0/(nbNodes*nbNodes)*100,totalTsentm*1.0/totalRecvm]
end

def sampleMeanAndSampleMeanStdev(population)
  n = 0
  mean = 0.0
  s = 0.0
  population.each { |x|
    n = n + 1
    delta = x - mean
    mean = mean + (delta / n)
    s = s + delta * (x - mean)
  }
  # if you want to calculate std deviation
  # of a sample change this to "s / (n-1)"
  return [mean, Math.sqrt(s / ((n-1)*n))]
end


def computeCoveragePoint(params)
  protocol  = params[:protocol]
  topo      = params[:topo]
  w         = params[:w]
  h         = params[:h]
  b         = params[:b]
  n         = params[:n]
  interval  = params[:interval]
  threshold = params[:threshold]
  phases    = params[:phases]
  num_measures = params[:num_measures]

  coverage_vec      = []
  costRatio_vec     = []
  for i in (0..num_measures-1)
    statsFile = getExperimentResults(protocol,threshold,topo,w,h,b,n,interval,i,phases)
    if phases.include?("-analysis") then
      (c,r) = extractCoverageAndCostRatioFromStatsFile(statsFile)
#      puts "#{c}\t#{r}\t#{statsFile}"
      coverage_vec  << c
      costRatio_vec << r
    end
  end
  if phases.include?("-analysis") then
    result = {
      :bridge    => b*2,
      :coverage  => sampleMeanAndSampleMeanStdev(coverage_vec),
      :costRatio => sampleMeanAndSampleMeanStdev(costRatio_vec)
    }
  else
    result = nil
  end
  if phases.include?("-distribution") then
    return coverage_vec
  end
  if phases.include?("-distCost") then
    return costRatio_vec
  end
  return result
end

def parseOptOrg(argv,argHash)
  for i in (0..argv.length-1)
    if (argv[i] =~ /^-/) then
      if !argHash.key?(argv[i]) then
        raise "Unknown option: #{argv[i]}"
      end
      if (i<argv.length-1) and not (argv[i+1] =~ /^-/) then
        argHash[argv[i]]=argv[i+1]
      else
        argHash[argv[i]]=true
      end
    end
  end
  return argHash
end

def outputResultVector(name,vector,protocol,argHash)
  if argHash['-autoout'] then
    file = ("#{protocol}-B#{argHash['-bridgeMin']}-#{argHash['-bridgeMax']}-" + \
            "STEP#{argHash['-bridgeStep']}-#{name}-plot.txt")
    $stderr.puts "Writing to #{file}"
    $stdout.reopen(file,"w")
  end
  vector.each{|line| puts line.join("\t") }
end

if __FILE__ == $0
  argHash = {
    "-proto"     => "OR", 
    "-nnodes"    => 500 ,
    "-width"     => 500 ,
    "-height"    => 200 ,
    "-bridgeMin" => 0   ,
    "-bridgeMax" => 20  ,
    "-bridgeStep"=> 2   ,
    "-test"      => false,
    "-redirect"  => false,
    "-out"       => false,
    "-autoout"   => false,
    "-spec"      => false,
    "-ns2"       => false,
    "-analysis"  => false,
    "-noorder"   => false,
    "-measures"  => 10,
    "-interval"  => 20,
    "-threshold" => 2
  }
  parseOptOrg(ARGV,argHash)
  if argHash["-test"] then 
    thisIsATest()
  end
  if argHash["-redirect"] then
    $stdout.reopen(argHash['-redirect'],"w")
    $stdout.sync=true
    $stderr.reopen($stdout)
  end
  if argHash["-out"] then
    $stdout.reopen(argHash['-out'],"w")
  end
  $stderr.puts argHash.inspect
  phaseS = [] # By default we don't create specs, we don't run any
              # simulation
  ["-spec", "-ns2", "-analysis"].each do|phase|
    if argHash[phase] then phaseS << phase end
  end
  # We don't log if only analysing
  if ((phaseS.include?("-spec") or 
       phaseS.include?("-ns2")  or 
       phaseS.include?("-out")) and
      !phaseS.include?("-test")) then 
    logExecution()
  end

  specialAnalysis = false

  if argHash['-analysis'] =~ /^dist.*B/ then
    scan   = argHash['-analysis'].scan(/^(dist[^0-9]*)B([0-9]*)$/).first
    type   = scan[0]
    bridge = scan[1].to_i
    argHash['-bridgeMin'] = bridge
    argHash['-bridgeMax'] = bridge
    case type
    when "dist"
      phaseS << "-distribution"
    else
      phaseS << "-" + type
    end
    specialAnalysis = true
  end

  argHash["-proto"].split(',').each do|protocol|

    allResults = {
      :coverage  => [],
      :costRatio => []
    }

    bridge = argHash['-bridgeMin'].to_i
    while bridge <= argHash['-bridgeMax'].to_i
      result = computeCoveragePoint(:protocol=>"policy-#{protocol}",
                                    :topo         =>"BRIDGE",
                                    :w            =>argHash['-width'    ].to_i,
                                    :h            =>argHash['-height'   ].to_i,
                                    :b            =>bridge,
                                    :n            =>argHash['-nnodes'   ].to_i,
                                    :interval     =>argHash['-interval' ].to_i,
                                    :num_measures =>argHash['-measures' ].to_i,
                                    :threshold    =>argHash['-threshold'].to_i,
                                    :phases       =>phaseS)
      if phaseS.include?("-analysis") and !specialAnalysis then 
        # puts "#{result[:bridge]}\t#{result[:coverage][0]}\t#{result[:coverage][1]}" 
        allResults[:coverage ] << [result[:bridge],result[:coverage ][0],result[:coverage ][1]]
        allResults[:costRatio] << [result[:bridge],result[:costRatio][0],result[:costRatio][1]]
      end

      if specialAnalysis then
        if argHash['-autoout'] then
          tag = ""
          if argHash['-noorder'] then
            tag = "-noorder"
          end
          file = "#{protocol}-#{argHash['-analysis']}#{tag}.txt"
          $stderr.puts "Writing to #{file}"
          $stdout.reopen(file,"w")
        end
        if !argHash['-noorder'] then
          result.sort!.reverse!
        end
        puts result.join("\n")
      end

      bridge += argHash['-bridgeStep'].to_i
    end # while bridge

    if phaseS.include?("-analysis") and !specialAnalysis then
      outputResultVector("coverage",allResults[:coverage ],protocol,argHash)
      outputResultVector("cost"    ,allResults[:costRatio],protocol,argHash)
    end

  end # do|protocol|
end
