# base values

set val(chan)       Channel/WirelessChannel
set val(prop)       Propagation/TwoRayGround
set val(netif)      Phy/WirelessPhy       
set val(mac)        Mac/802_11
set val(ifq)        Queue/DropTail/PriQueue
set val(ll)         LL
set val(ant)        Antenna/OmniAntenna
set val(ifqlen)         20                     ;# max packet in ifq
set val(seed)           0.0

#set val(x) 				800
#set val(y) 				600

set val(x1)				500
set val(x2)				1000

#set val(y1)				600
#set val(y2)				900

LL set mindelay_                50us
LL set delay_                   25us

Queue/DropTail/PriQueue set Prefer_Routing_Protocols    1

# unity gain, omni-directional antennas
# set up the antennas to be centered in the node and 1.5 meters above it
Antenna/OmniAntenna set X_ 0
Antenna/OmniAntenna set Y_ 0
Antenna/OmniAntenna set Z_ 1.5
Antenna/OmniAntenna set Gt_ 1.0
Antenna/OmniAntenna set Gr_ 1.0
                                                                                
# Initialize the SharedMedia interface with parameters to make
# it work like the 914MHz Lucent WaveLAN DSSS radio interface
Phy/WirelessPhy set CPThresh_ 10.0
Phy/WirelessPhy set CSThresh_ 1.559e-11

puts "TXPower: [Phy/WirelessPhy set Pt_]"
Phy/WirelessPhy set Rb_ 2*1e6
Phy/WirelessPhy set Pt_ 0.2818
Phy/WirelessPhy set freq_ 914e+6
Phy/WirelessPhy set L_ 1.0

Phy/WirelessPhy set RXThresh_ 1.20158e-07
#  ./a.out -m TwoRayGround -Pt 0.2818 -fr 914e+6 -Gt 1.0 -Gr 1.0 -L 1.0 -ht 1.5 -hr 1.5 250
# distance = 250
# propagation model: TwoRayGround

# Selected parameters:
# transmit power: 0.2818
# frequency: 9.14e+08
# transmit antenna gain: 1
# receive antenna gain: 1
# system loss: 1
# transmit antenna height: 1.5
# receive antenna height: 1.5

# Receiving threshold RXThresh_ is: 3.65213e-10

# Other values:
# 20m: 4.80631e-05
# 40m: 1.20158e-07
# 50m: 7.69009e-08
# 75m: 3.41782e-08
# 100m: 1.42661e-08
# 125m: 5.8434e-09
# 150m: 2.818e-09
# 175m: 1.52109e-09
# 200m: 8.91633e-10
# 225m: 5.56642e-10
# 250m: 3.65213e-10

Mac/802_11 set dataRate_ 2Mb



set val(adhocRouting)   NOAH
#set val(nn)             100                ;# how many nodes are simulated
#set val(stop)           1000              ;# simulation time

# Default output value
set val(out)            "pampa-output"
set val(usenam)         1
set val(verbose)        false


# load command-line options

for {set i 0} {$i < $argc} {incr i} {
    set arg [lindex $argv $i]
    if {[string range $arg 0 0] != "-"} continue
    
    set name [string range $arg 1 end]
    set val($name) [lindex $argv [expr $i+1]]
}

if { $val(verbose) } {
    puts "DataRate: [Mac/802_11 set dataRate_]"
    puts "Range: $val(range) RXThresh: [Phy/WirelessPhy set RXThresh_]"
}

# =====================================================================
# Main Program
# ======================================================================

#
# Initialize Global Variables
#

# [CHANGE] FT25Jul09: Removing all uneeded packet headers as explained
# in tcl/lib/ns-packet.tcl

# remove-all-packet-headers
# add-packet-header IP Mac Pampa

# create simulator instance

set ns_		[new Simulator]

# setup topography object

set topo	[new Topography]

# create trace object for ns and nam

set tracefd	[open $val(out).tr w] 
$ns_ use-newtrace
$ns_ trace-all $tracefd

if {$val(usenam)==1} {
    #set namtrace    [open $val(out).nam w]
    #$ns_ namtrace-all-wireless $namtrace $val(x) $val(y)
}

# define topologyQ
$topo load_flatgrid $val(x) $val(y)

#
# Create God
#
set god_ [create-god $val(nn)]

#
# define how node should be created
#

#global node setting

set chan [new $val(chan)]

$ns_ node-config \
    -adhocRouting $val(adhocRouting) \
    -llType $val(ll) \
    -macType $val(mac) \
    -ifqType $val(ifq) \
    -ifqLen $val(ifqlen) \
    -antType $val(ant) \
    -propType $val(prop) \
    -phyType $val(netif) \
    -channel $chan \
    -topoInstance $topo \
    -agentTrace OFF\
    -routerTrace OFF \
    -macTrace ON  


#
#  Create the specified number of nodes [$val(nn)] and "attach" them
#  to the channel. 

for {set i 0} {$i < $val(nn) } {incr i} {
    set node_($i) [$ns_ node]	
    $node_($i) random-motion 0	;# disable random motion
    set pampa_($i) [new Agent/PampaAgent]
    $node_($i) attach $pampa_($i) 3323
    $ns_ attach-agent $node_($i) $pampa_($i) 

    $pampa_($i) $val(optpampa) $val(optargs); # $val(optarg2);

    # Set the destination for log info
    set T [new Trace/Generic]
    $T target [$ns_ set nullAgent_]
    $T attach $tracefd
    $T set src_ [$node_($i) id]
    $pampa_($i) log-target $T
}

set finalstatsfd [open $val(out).stats w]

#
# Creates the Trace object to flush cache state messages. Several 
# can be created and each can be passed to a different instance of
# the dump-state command
#

set finalstats [new Trace/Generic]
$finalstats target [$ns_ set nullAgent_]
$finalstats attach $finalstatsfd

#
# Load a movement file
#

#source $val(mov)

#
# Placing the nodes at the correct position for nam
#

source $val(pos)

#
# Some pampa commands
#

#$ns_ at 1.0 "$pampa_(0) bcast 1000"

source $val(traf)

#
# Dump results
#
for {set i 0} {$i < $val(nn) } {incr i} {
    $ns_ at [expr $val(stop)+0.1] "$pampa_($i) dump-stats $finalstats";
}

if { $val(verbose) } {
	$ns_ at $val(stop) "puts \"NS EXITING...\" ; $ns_ halt"
}

puts $tracefd "M 0.0 nn $val(nn) x $val(x) y $val(y) rp $val(adhocRouting)"
puts $tracefd "M 0.0 prop $val(prop) ant $val(ant)"

#puts $tracefd "[$pampa_(0) print-config]"

if { $val(verbose) } {
        puts "NS Starting... with $val(pos) and $val(traf)"
}

$ns_ run

