#!/usr/bin/ruby -w

require 'socket'

EXPERIENCE_SPEC_DIRECTORY="EXP_SPECS"

$DIR_PREFIX =""

def thisIsATest()
  $DIR_PREFIX="TEST_"
end

def getTimeStamp()
  time = Time.new
  host = Socket.gethostname.gsub!(/[[:space:].]/,"")
  return "%04d%02d%02d%02d%02d-AT%s" % 
    [time.year, time.month, time.day, time.hour, time.min, host]
end

# ************************************************
# w,h: width and height of the simulation
# b: half-size of the bridge (height direction)
# n: number of nodes before the bridge is created, i.e. n/(w*h) is the
# node density
# i: interval between broacast
def paramString(topo,w,h,b,n,i)
  return "T-#{topo}-W#{w}-H#{h}-N#{n}-B#{b}-I#{i}"
end

# ************************************************
# w,h: width and height of the simulation
# b: half-size of the bridge (height direction)
# n: number of nodes before the bridge is created, i.e. n/(w*h) is the
# node density
# i: interval between broacast

def experimentString(topo,w,h,b,n,i,id)
  return "#{paramString(topo,w,h,b,n,i)}-ID#{id}"
end

# ************************************************
# w,h: width and height of the simulation
# b: half-size of the bridge (height direction)
# n: number of nodes before the bridge is created, i.e. n/(w*h) is the
# node density
# i: interval between broacast

def experimentSpecName(topo,w,h,b,n,i,id)
  return "#{getTimeStamp()}-#{experimentString(topo,w,h,b,n,i,id)}"
end

# ************************************************
# w,h: width and height of the simulation
# b: half-size of the bridge (height direction)
# n: number of nodes before the bridge is created, i.e. n/(w*h) is the
# node density

def createPositionFileWithBridge(fileName,w,h,b,n)
  nodePositions = []
  for i in (1..n)
    x = rand() * w
    y = rand() * h
    if  (x>=w/3.0) and (x<=2*w/3.0) and
        ((h/2.0-y).abs>=b) then next end
    nodePositions << [x,y]
  end


  if File.exist?(fileName) then raise "#{fileName} already exists." end
  out = File.new(fileName, "w")
  out.puts "# NNODES: #{nodePositions.length}, WIDTH: #{w}, HEIGHT: #{h}, HALF_WIDTH_BRIDGE: #{b} NNODES_WITHOUT_BRIDGE: #{n}"

  count = 0;
  nodePositions.each do|pos|
    out.puts "$node_(#{count}) set X_ #{pos[0]}"
    out.puts "$node_(#{count}) set Y_ #{pos[1]}"
    count+=1
  end
  out.close
  return count
end

# ************************************************
# nnodesWithBridge: nodes once the bridge is there
# w,h: width and height of the simulation
# b: half-size of the bridge (height direction)
# n: number of nodes before the bridge is created, i.e. n/(w*h) is the
# node density
# interval: interval between broacast

def createTrafficFile(fileName,nnodesWithBridge,w,h,b,n,interval,id)
  if File.exist?(fileName) then raise "#{fileName} already exists." end
  out = File.new(fileName, "w")
  out.puts("# " \
           +                "NNODES: #{nnodesWithBridge}, " \
           +                 "WIDTH: #{w}, "                \
           +                "HEIGHT: #{h}, "                \
           +     "HALF_WIDTH_BRIDGE: #{b}, "                \
           + "NNODES_WITHOUT_BRIDGE: #{n}, "                \
           +              "INTERVAL: #{interval},")
  time = 0.0
  for i in (0..nnodesWithBridge-1)
    out.puts "$ns_ at #{time} \"$pampa_(#{i}) bcast 1000\""
    time += interval
  end
  out.close
end

# ************************************************
# w,h: width and height of the simulation
# b: half-size of the bridge (height direction)
# n: number of nodes before the bridge is created, i.e. n/(w*h) is the
# node density
# interval: interval between broacast

def createExperimentSpec(topo,w,h,b,n,interval,id,
                         directory=EXPERIENCE_SPEC_DIRECTORY)
  specName  = 
    "#{$DIR_PREFIX}#{directory}/"+\
    "#{experimentSpecName(topo,w,h,b,n,interval,id)}"
  puts "--> Creating specification files for "+\
    "#{experimentString(topo,w,h,b,n,interval,id)}"

  nnodesWithBridge = 
    case topo
    when "BRIDGE" then
      createPositionFileWithBridge("#{specName}-position.txt",w,h,b,n)
    else 
      raise "I don't know how to create topology #{topo}"
    end
  createTrafficFile("#{specName}-traffic.txt",
                    nnodesWithBridge,w,h,b,n,interval,id)
  return [nnodesWithBridge,specName]
end

def provideDirectory(directory)
  if !(File.directory? directory) then
    Dir.mkdir directory
  end
end

# ************************************************
# w,h: width and height of the simulation
# b: half-size of the bridge (height direction)
# n: number of nodes before the bridge is created, i.e. n/(w*h) is the
# node density
# interval: interval between broacast

def getExperimentSpecFiles(topo,w,h,b,n,i,id,
                           phases,
                           directory=EXPERIENCE_SPEC_DIRECTORY)
  provideDirectory("#{$DIR_PREFIX}#{directory}")
  positionFileS = 
    Dir.glob("#{$DIR_PREFIX}#{directory}/*"+\
             "#{experimentString(topo,w,h,b,n,i,id)}-position.txt")
  trafficFileS = 
    Dir.glob("#{$DIR_PREFIX}#{directory}/*"+\
             "#{experimentString(topo,w,h,b,n,i,id)}-traffic.txt")
  case
  when ((positionFileS.length==0) and (trafficFileS.length==0)) then
    # None exists
    if phases.include?("-spec") then
      (nnodesWithBridge,specName) = 
        createExperimentSpec(topo,w,h,b,n,i,id,directory)
      positionFile = "#{specName}-position.txt"
      trafficFile  = "#{specName}-traffic.txt"
    else
      if phases.include?("-ns2") or phases.include?("-analysis") then
        raise("Cannot run simulation or do analysis: spec file missing "+\
              "for #{experimentString(topo,w,h,b,n,i,id)}. "+\
              "Allow -spec phase to solve this")
      end
    end
  when ((positionFileS.length==1) and (trafficFileS.length==1)) then
    # Both exist
    positionFile = positionFileS.first
    trafficFile  = trafficFileS .first
    f = File.open(positionFile, "r")
    nnodesWithBridge = f.gets.scan(/# NNODES: ([0-9]*),/).first.first.to_i
    specName = positionFile.scan(/^(.*)-position.txt/).first.first
    f.close
  else
    raise("Too many or not enough traffic and position files!")
  end
  if (specName) then
    specName = File.basename(specName)
  end
  return [nnodesWithBridge,positionFile,trafficFile,specName]
end

if __FILE__ == $0
# This no longer works
#   (nn,positionFile,trafficFile,specName) = 
#     getExperimentSpecFiles(:topo=>"BRIDGE",:w=>300,:h=>40,:b=>6,:n=>60,:i=>20,:id=>0)
#   system "time ns test.tcl -traf \"#{positionFile}\" -pos \"#{trafficFile}\" -optpampa policy-delpwr -optargs \"3e6 2\" -nn #{nn} -x 40 -y 300 -stop 20000 -out pampa-francois-output"

#  puts experimentSpecName(500,200,2,500)
end

